#include <hidef.h>				// for EnableInterrupts macro
#include "derivative.h"			// include peripheral declarations
#include "LIB_ICS.h"				// ICS routines 
//#include "LIB_SCI.h"				// SCI routines


#define SW1 PTHD_PTHD6
#define SW2 PTHD_PTHD7
#define SW3 PTHD_PTHD4
#define SW4 PTFD_PTFD0
#define SW5 PTHD_PTHD0
#define SW6 PTHD_PTHD1
#define SW7 PTHD_PTHD2
#define SW8 PTHD_PTHD3


#define LED1 PTHD_PTHD5
#define LED2 PTHD_PTHD4
#define LED3 PTID_PTID5
#define LED4 PTID_PTID4
#define LED5 PTID_PTID3
#define LED6 PTFD_PTFD5
#define LED7 PTFD_PTFD4
#define LED8 PTFD_PTFD3

#define ON		0
#define OFF		1


void All_Leds_On(void);
void All_Leds_Off(void);
void sw_debounce(void);
byte Get_SW_Status(void);

byte buz_st = 0;

void main(void) {
	
	SOPT1 = 0x23;					// disable COP, RSTPE, BKGPE
	EnableInterrupts;
	ICS_FEE_32K_8M();				// FEE mode, 8MHz busclk from 32kHz input
		
	// setup clock gating
	SCGC1_ADC = 1;					// enable clock to ADC module
	SCGC1_TPM2 = 1;				//	enable clock to TPM2	module
	SCGC2_KBI = 1;					// enable clock to KBI module

	// setup KBI
	PINPS1 = 0xF7;					// setup KBI pin select input
	KBIPE = 0xFF;					// KBI[7:0] are inerrupt inputs
	KBISC_KBACK = 1;				// clear any pending interrupt
	KBISC_KBIE = 1;				// enable keyboard interrupts

	
	// setup port IO

	PTFD = 0x38;					// PTF[5:3]	high
	PTFDD = 0x38;					// PTF[5:3] output

	PTID = 0x38;					// PTI[5:3]	high
	PTIDD = 0x38;					// PTI[5:3] output

	PTHD = 0x20;					// PTH5	high
	PTHDD = 0x20;					// PTH5 output
	
	PTHPE = 0xDF;					// pull en on PTH[7:6,4:0]
	PTFPE = 0x01;					// pull en on PTF0

	All_Leds_On();
	All_Leds_Off();				// all LEDs OFF
	
	// setup TPM2
	TPM2SC_CLKSx = 0x00;			// disable TPM2 counter
	TPM2C5SC_MS5x = 0x01;		// config TPM2 Ch5 for output compare
	TPM2C5SC_ELS5x = 0x01;		// toggle on compare
	TPM2C5V = 0x0360;				// load ch5 count - output ~1200 Hz
	TPM2SC_PS = 0x2;				// input clk div 4
	if(TPM2C5SC_CH5F)
		TPM2C5SC_CH5F = 0;		// clear channel flag if set
	TPM2C5SC_CH5IE = 1;			// enable TPM interrupt
	

	// setup ADC
	ADCCFG_ADIV = 0x02;			// input clk div 2 (4MHz)
	ADCCFG_MODE = 0;				// 8bit results mode
	ADCCFG_ADICLK = 0;			// clk input from busclk
	APCTL1_ADPC6 = 1;				// select POT input to ADC
	ADCSC1 = 0x66;					// en ADC int on convert complete
										// set for continuous conversion
										// select ADC CH6
	for(;;){}
}

void All_Leds_Off(void){
	PTHD |= 0x30;					// LED[2:1] OFF
	PTID |= 0x38;					// LED[5:3] OFF
	PTFD |= 0x38;					// LED[8:6] OFF
}


void All_Leds_On(void){
	PTHD &= 0xCF;					// LED[2:1] ON
	PTID &= 0xC7;					// LED[5:3] ON
	PTFD &= 0xC7;					// LED[8:6] ON
}


byte Get_SW_Status(void){
	if(!SW1) return 1;
	if(!SW2) return 1;
	if(!SW3) return 1;
	if(!SW4) return 1;
	if(!SW5) return 1;
	if(!SW6) return 1;
	if(!SW7) return 1;
	if(!SW8) return 1;
	
	return 0;
}


void sw_debounce(void){
   while(Get_SW_Status());		// wait for all switches to be released
   while(Get_SW_Status());		// wait for all switches to be released
}

void interrupt 16 kbi_isr(void){

	byte pth, ptf;
	KBISC_KBIE = 0;				// disable keyboard interrupts
	pth = PTHD;						// get PTH state
	ptf = PTFD;						// get PFT state

	if(!(pth & 0x80)){
		while(!SW2);					// debounce SW2
		while(!SW2);					// debounce SW2
		ADCSC1 = 0x00;				   // disable ADC 
		APCTL1_ADPC6 = 0;				// disable ADC input
		pth |= 0x01;					// set PTH0
		All_Leds_Off();				// all LEDs off
	}
		
	
	sw_debounce();
	
	if(!(pth & 0x40)) LED1 ^= 1;	// toggle LED1 if SW1 pressed	
	if(!(pth & 0x10)) LED3 ^= 1;	// toggle LED3 if SW3 pressed
	if(!(ptf & 0x01)) LED4 ^= 1;	// toggle LED4 if SW4 pressed
	if(!(pth & 0x01)) LED5 ^= 1;	// toggle LED5 if SW5 pressed
	if(!(pth & 0x02)) LED6 ^= 1;	// toggle LED6 if SW6 pressed
	if(!(pth & 0x04)) LED7 ^= 1;	// toggle LED7 if SW7 pressed
	if(!(pth & 0x08)){				// if SW8 pressed
		TPM2SC_CLKSA ^= 1;			// toggle TPM2 clk input
	}

	KBISC_KBACK = 1;					// clr int flag
	KBISC_KBIE = 1;					// enaable keyboard interrupts
}


void interrupt 17 adc_isr(void){
	byte adc;
	ADCSC1_AIEN = 0;					// disable ADC interrupt
	adc = (ADCRL / 0x1C);			// get ADC reading
	All_Leds_Off();					// all leds off
	
	switch(adc){
		case 9:
		case 8:	LED8 = ON;			// led8 on 
		case 7:	LED7 = ON;			// led7 on 
		case 6:	LED6 = ON;			// led6 on 
		case 5:	LED5 = ON;			// led5 on 
		case 4:	LED4 = ON;			// led4 on 
		case 3:	LED3 = ON;			// led3 on 
		case 2:	LED2 = ON;			// led2 on 
		case 1:	LED1 = ON;			// led1 on 
				 	break;
		case 0:	All_Leds_Off();		// all leds off 
				 	break;
		default: All_Leds_Off();		// all leds off 
					break;
	}
	
	ADCSC1_AIEN = 1;					// enable ADC interrupt
	adc = ADCRL;						// dummy read to clear missed int
}


void interrupt 23 tpm2c5_isr(void){
	
	TPM2C5SC_CH5F = 0;				// clear channel flag
	TPM2CNTL = 0;						// clear count
	
}

